;nyquist plug-in
;version 4
;type process
;name Beeper
;author "Steve Daulton"
;release 3.1.3
;copyright "GNU General Public License v2.0 or later"


;;; Add "spoiler" beeps (watermark) to a track.

;control hz "Beep frequency" int "Hz" 1000 0 10000
;control harmonics "Beep harmonics" float "%" 10 0 100
;control dur "Beep duration" float "seconds" 0.5 0 10
;control variation "Beep variation" float "%" 25 0 100
;control gain "Beep level" float "0 to 1" 0.5 0 1
;control interval "Interval" float "seconds" 30 0 100
;control randomize "Interval randomization" float "%" 0 0 100
;control mode "Replace or overlay" choice "Replace,Overlay" 1



(defmacro range-adjust (param oldmin oldmax &optional (newmin 0) (newmax 1))
  ;; Adjust range of input to output range.
  `(let ((oldrange (- ,oldmax ,oldmin))
         (newrange (- ,newmax ,newmin)))
    (setf ,param (/ (- ,param ,oldmin) (float oldrange)))
    (setf ,param (+ (* ,param (float newrange)) ,newmin))))


(range-adjust harmonics 0 100 1 10)
(range-adjust variation 0 100)
(range-adjust randomize 0 100)


(defun tone (pitch seconds delta)
  (let ((tone (osc pitch seconds)))
    (setf tone (clip (mult harmonics tone) 1))
    (setf tone (lowpass2 tone (step-to-hz pitch)))
    (mult (pwl delta 1 (- seconds delta) 1 seconds 0)
          tone)))


(defun randpitch ()
  ;; 100% variation => 1 octave
  (let ((rand (* 12 variation (- (rrandom) 0.5))))
    (+ (hz-to-step hz) rand)))


(defun randdur (var amount)
  ;; 100% variation => 0.5 to 2.0 times var
  (let ((rand (* amount  (- (* 2 (rrandom)) 1))))
    (* var (power 2.0 rand))))


(defun add-breakpoints (bp gap tonedur delta)
  (push gap bp)
  (push 1 bp)
  (push delta bp)
  (push 0 bp)
  (push (- tonedur (* 2 delta)) bp)
  (push 0 bp)
  (push delta bp)
  (push 1 bp))


(let ((delta (min 0.05 (/ dur 4)))
      (totaldur 0)
      (breakpoints '(1 0))
      (beeps (s-rest 1)))
  (do ((tonedur (randdur dur variation) (randdur dur variation))
       (silence (randdur interval randomize) (randdur interval randomize)))
      ((>= totaldur (get-duration 1)))
    (when (= mode 0)
      (setf breakpoints (add-breakpoints breakpoints silence tonedur delta)))
    (let ((beep (stretch-abs 1 (tone (randpitch) tonedur delta))))
      (setf beeps (sim beeps
                       (at-abs (+ totaldur silence) (cue beep)))))
    (setf totaldur (+ totaldur tonedur silence)))
  (setf beeps (extract 0 1 beeps))
  (when (arrayp *track*)
    (setf beeps (vector beeps beeps)))
  (if (= mode 0)  ;Replace audio with beeps
    (let ((env (stretch-abs 1 (pwlr-list (reverse breakpoints)))))
      (sum (mult *track* env)
           (mult gain beeps)))
    (sum *track* (mult gain beeps))))
